@@ -1,9 +1,56 @@
+// XXX Wrap this class.
+// (function() {
+
 Test.Base = function() {
     this.init.apply(this, arguments);
 }
-proto = Test.Base.prototype;
+var proto = Test.Base.prototype;
 
-Test.Base.VERSION = '0.14';
+Test.Base.VERSION = '0.15';
+
+Test.Base.newSubclass = function(name, base, isFinal) {
+    if (
+        !name || typeof(name) != 'string' ||
+        base && typeof(base) != 'string'
+    ) throw "Usage: Test.Base.newSubclass('subclass.name' [, 'baseclass.name'])";
+    if (!base)
+        base = 'Test.Base';
+
+    var parts = name.split('.');
+    var subclass = window;
+    for (var i = 0; i < parts.length; i++) {
+        if (! subclass[parts[i]])
+            subclass[parts[i]] = function() {
+                try { this.init() } catch(e) {throw(e)}
+            };
+        subclass = subclass[parts[i]];
+    }
+
+    var baseclass = eval('new ' + base + '()');
+    subclass.prototype = baseclass;
+    subclass.prototype.className = name;
+    subclass.prototype.baseClassName = base;
+
+    if (! isFinal) {
+        subclass.prototype.init = function() {
+            eval(base).prototype.init.call(this);
+            this.block_class = name + '.Block';
+        }
+
+        var block_proto =
+            Test.Base.newSubclass(name + '.Block', base + '.Block', true);
+
+        block_proto.init = function() {
+            eval(base + '.Block').prototype.init.call(this);
+            this.filter_object = eval('new ' + name + '.Filter()');
+        }
+
+        Test.Base.newSubclass(name + '.Filter', base + '.Filter', true);
+    }
+
+    subclass.prototype.classname = name;
+    return subclass.prototype;
+}
 
 proto.init = function() {
     this.builder = Test.Builder.instance();
@@ -41,6 +88,7 @@ proto.run_is = function(x, y) {
     }
 }
 
+// TODO Add more proxy functions for common builder stuffs
 proto.plan = function(number) {
     var cmds = {tests: number};
     return this.builder.plan(cmds);
@@ -54,6 +102,10 @@ proto.fail = function(name) {
     return this.builder.ok(false, name);
 }
 
+proto.ok = function(test, desc) {
+    this.builder.ok(test, desc);
+}
+
 proto.is = function (got, expect, desc) {
     return this.builder.isEq(got, expect, desc);
 };
@@ -70,6 +122,14 @@ proto.unlike = function (val, regex, desc) {
     return this.builder.unlike(val, regex, desc);
 };
 
+proto.skip = function(why) {
+    return this.builder.skip(why);
+};
+
+proto.skipAll = function(why) {
+    return this.builder.skipAll(why);
+};
+
 proto.compile = function() {
     if (this.state.compiled) return;
     this.get_spec();
@@ -127,7 +187,7 @@ proto.make_block = function(hunk) {
         if (index < 0) throw('xxx1');
         var line1 = chunk.substr(0, index);
         var section_data = chunk.substr(index + 1);
-        line1 = line1.replace(/^---\s*/, '');
+        line1 = line1.replace(/^---\s+/, '');
         if (! line1.length) throw('xxx2');
         var section_name = '';
         var section_filters = [];
@@ -201,7 +261,10 @@ proto.filter_section = function(section, filters) {
     var data = this.data[section];
     for (var i = 0; i < filters.length; i++) {
         var filter = filters[i];
-        if (typeof window[filter] == 'function')
+        if (typeof filter == 'function') {
+            data = filter.call(this, data, this);
+        }
+        else if (typeof window[filter] == 'function')
             data = (window[filter]).call(this, data, this);
         else if (typeof this.filter_object[filter] == 'function')
             data = (this.filter_object[filter]).call(this, data, this);
@@ -249,10 +312,11 @@ function XXX(msg) {
     //if (! confirm(arguments.join('\n')))
     if (! confirm(msg))
         throw("terminated...");
+    return msg;
 }
 
 function JJJ(obj) {
-    XXX(JSON.stringify(obj));
+    return XXX(JSON.stringify(obj));
 }
 
 //------------------------------------------------------------------------------
